from org.processmining.ptandloggenerator.interfaces import DataLogInterface

import gc
from tree_with_data_dependencies import TreeWithDataDependencies
from simulate_tree_with_rules import LogSimulator

class DataExtendLogSimulator(DataLogInterface):

    def __init__(self,*args):
        # garbage collection
        gc.collect()
        # handle arguments
        newick_tree_string = args[0]
        no_cases = int(args[1])
        no_noisy_cases = int(args[2])
        max_input = int(args[3])
        max_cutoff = int(args[4])
        determinism = float(args[5])
        # initialize tree with data extension
        tree_w_data = TreeWithDataDependencies(newick_tree_string,
                                               target_dl=determinism)
        # if the tree has at least one choice, then we can add data dependencies
        if tree_w_data.data_dependencies_possible:
            # extend with data dependencies
            tree_w_data.extend_tree_with_data_dependencies(max_input,max_cutoff)
            # initialize log simulator
            simulator = LogSimulator(tree_w_data.t,
                                     tree_w_data.input_choice_dictionary,
                                     tree_w_data.rules_simulation,
                                     tree_w_data.case_attr,
                                     False)
            # simulate tree
            simulator.simulate(no_cases)
            # simulate noisy traces
            simulator.simulate_noise(no_cases, no_noisy_cases, tree_w_data.removed_rules_simulation)
            self.log = simulator.log.cases
        else:
            print 'no data dependencies possible!'

    def returnLog(self):
        reformatted_log = []
        for case in self.log:
            case_attr_dict = dict()
            for attr in case.case_attrs:
                case_attr_dict[attr.name] = attr.value
            new_case = [case.trace,case_attr_dict]
            reformatted_log.append(new_case)
        return reformatted_log