from org.processmining.ptandloggenerator.interfaces import uniqueTraces

from tree import TreeNode
import itertools
import gc
from math import factorial as fac

def binomial(x, y):
    try:
        binom = fac(x) / fac(y) / fac(x - y)
    except ValueError:
        binom = 0
    return binom

def simplify(paths):
    results = []
    d = dict(paths)
    keys = d.keys()
    for key in keys:
        matches = [item for item in paths if item[0] == key]
        x_0 = 0
        for (z_i, x_i) in matches:
            x_0 += x_i
        results.append((z_i, x_0))
    return results

def sequence(r,path):
    results = []
    for (z_r, x_r) in r:
        for (z_i, x_i) in path:
            x_0 = x_r * x_i
            z_0 = z_r + z_i
            results.append((z_0,x_0))
    return results

def choice(r,path):
    results = r
    for (z_i, x_i) in path:
        results.append((z_i, x_i))
    return results

def parallel(r,path):
    results = []
    for (z_r, x_r) in r:
        for (z_i, x_i) in path:
            x_0 = x_r * binomial(z_r + z_i, z_r) * x_i * binomial(z_i, z_i)
            z_0 = z_r + z_i
            results.append((z_0,x_0))
    return results

def loop(paths,iterations):
    do = paths[0]
    redo = paths[1]
    exit_ = paths[2]
    repeat = [(0,1)]
    xor_set = repeat
    for i in range(iterations):
        repeat = sequence(sequence(repeat,redo),do)
        xor_set = choice(xor_set,repeat)
    results = sequence(sequence(do,xor_set),exit_)
    return results

def or_(paths):
    results = []
    for i in range(1,len(paths)+1):
        for c in itertools.combinations(paths,i):
            if len(list(c)) < 2:
                for (z_i, x_i) in c[0]:
                    results.append((z_i, x_i))
            else:
                r = c[0]
                for i in range(1,len(c)):
                    r = parallel(r,c[i])
                for (z_i, x_i) in r:
                    results.append((z_i, x_i))
    return results

def numberOfPaths(node,paths,k):
    if (node.is_leaf()):
        if node.name == "tau":
            return [(0,1)]
        else:
            return [(1,1)]

    else:
        paths = []
        for child in node.get_children():
            paths.append(numberOfPaths(child,paths,k))
        if node.name == "sequence":
            results = paths[0]
            for i in range(1,len(node.get_children())):
                results = sequence(results,paths[i])
            #print node.name, simplify(results)
            return simplify(results)

        elif node.name == "choice":
            results = paths[0]
            for i in range(1,len(node.get_children())):
                results = choice(results,paths[i])
            #print node.name, simplify(results)
            return simplify(results)

        elif node.name == "parallel":
            results = paths[0]
            for i in range(1,len(node.get_children())):
                results = parallel(results,paths[i])
            #print node.name, simplify(results)
            return simplify(results)

        elif node.name == "loop":
            results = loop(paths,k)
            #print node.name, simplify(results)
            return simplify(results)

        elif node.name == "or":
            results = or_(paths)
            #print node.name, simplify(results)
            return simplify(results)

def sumOfPaths(paths):
    total_paths = 0
    for (z_i, x_i) in paths:
        total_paths += x_i
    return total_paths


class NoUniqueTraces(uniqueTraces):
    def __init__(self,tree_string,loop_iterations):
        gc.collect()
        t = TreeNode(tree_string,format = 1)
        paths = []
        paths = numberOfPaths(t.get_tree_root(),paths,loop_iterations)
        self.no_unique_traces = sumOfPaths(paths)

    def getNoUniqueTraces(self):
        return self.no_unique_traces


#BENCHMARK#
"""
tree_path = "../trees_to_bpsim/plugins/Trees/ATAED2/tree_1_"
outfile = open("output_benchmark.csv","w")
outfile.write("tree,n\n")
for i in range(1,51):
    filepath = tree_path + str(i) + ".nw"
    t = TreeNode(filepath,format=1)
    n = NoUniqueTraces(t.write(format=1,format_root_node=True),1)
    outfile.write("tree_1_" + str(i) + "," + str(n.no_unique_traces) + "\n")

outfile.close()
"""