package org.processmining.ptandloggenerator.algorithms;

import org.processmining.ptandloggenerator.models.DataLogCollection;
import org.processmining.ptandloggenerator.models.NewickTree;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;
import org.python.modules.gc;

public class DataLogCollectionGenerator {
	
	/**
	 * creates a collection of logs with data (LogCollection) out of a collection of Newick trees
	 * (NewickTreeCollection)
	 */
	
	private DataLogCollection resultingLogs;
	private DataLogFactory factory = null;
	
	public DataLogCollectionGenerator(NewickTreeCollection trees, String noCases, 
			String noNoisyCases,String maxInput, String maxCutoff, String determinism){
		
		resultingLogs = new DataLogCollection();
		factory = new DataLogFactory();
		
		int i = 0;
		for (NewickTree tree: trees.getNewickTreeList()){
			// only extend and generate data logs for trees with at least one choice!
			String choice = "choice";
			if (tree.getTree().contains(choice)){
				DataLogGenerator generator = new DataLogGenerator(tree, noCases,
						noNoisyCases,maxInput, maxCutoff, determinism, factory);
				resultingLogs.addLogToCollection(generator.getResultingTraces());
				System.out.println("iteration: " + i++);
				factory.cleanupInterpreter();
			}
			else {
				System.out.println("tree has no choices -> NO LOG in iteration: " + i++);
			}
		}
	}
	
	public DataLogCollection getLogs(){
		return resultingLogs;
	}
	
	public void cleanUp()
	{
		factory.cleanupInterpreter();
		gc.collect();
	}

}
