package org.processmining.ptandloggenerator.algorithms;

import org.processmining.ptandloggenerator.interfaces.DataLogInterface;
import org.python.core.PyObject;
import org.python.core.PyString;
import org.python.core.PyStringMap;
import org.python.core.PySystemState;
import org.python.util.PythonInterpreter;

public class DataLogFactory {
	
	private PythonInterpreter interpreter;
	private PyObject jyLogClass;
	
	public DataLogFactory() {
		
		if (interpreter == null) {
			PythonInterpreter.initialize(System.getProperties(), null, new String[] {});
			interpreter = new PythonInterpreter(new PyStringMap(), new PySystemState());
		}
		initializeInterpreter();
	}
	
	private void initializeInterpreter() {
		interpreter.exec("from addDataSimulateTree import DataExtendLogSimulator");
		jyLogClass = interpreter.get("DataExtendLogSimulator");
	}
	
	public DataLogInterface createLog(String newickTree, String noCases, String noNoisyCases,
			String maxInput, String maxCutoff, String determinism) {
		PyObject[] args = new PyString[6];
		args[0] = new PyString(newickTree);
		args[1] = new PyString(noCases);
		args[2] = new PyString(noNoisyCases);
		args[3] = new PyString(maxInput);
		args[4] = new PyString(maxCutoff);
		args[5] = new PyString(determinism);
		//System.out.println(args);
		PyObject caseObj = jyLogClass.__call__(args);
		return (DataLogInterface) caseObj.__tojava__(DataLogInterface.class);
	}
	
	public void cleanupInterpreter() {
		interpreter.cleanup();
	}

}
