package org.processmining.ptandloggenerator.algorithms;

import org.processmining.ptandloggenerator.models.LogCollection;
import org.processmining.ptandloggenerator.models.NewickTree;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;
import org.processmining.ptandloggenerator.parameters.LogParameters;
import org.python.modules.gc;

public class LogCollectionGenerator {
	
	/**
	 * creates a collection of logs (LogCollection) out of a collection of Newick trees
	 * (NewickTreeCollection)
	 */
	
	private LogCollection resultingLogs;
	private LogFactory factory = null;
	
	public LogCollectionGenerator(NewickTreeCollection trees, int numTrainigTraces){
		
		resultingLogs = new LogCollection();
		LogParameters parameters = new LogParameters(numTrainigTraces);
		factory = new LogFactory();
		
		int i = 0;
		for (NewickTree tree: trees.getNewickTreeList()){
			LogGenerator generator = new LogGenerator(tree, parameters,factory);
			resultingLogs.addLogToCollection(generator.getResultingTraces());
			System.out.println("iteration: " + i++);
			factory.cleanupInterpreter();
		}
	}
	
	public LogCollection getLogs(){
		return resultingLogs;
	}
	
	public void cleanUp()
	{
		factory.cleanupInterpreter();
		gc.collect();
	}
}
