package org.processmining.ptandloggenerator.algorithms;

import org.processmining.ptandloggenerator.interfaces.randomTree;
import org.python.core.PyObject;
import org.python.core.PyString;
import org.python.core.PyStringMap;
import org.python.core.PySystemState;
import org.python.util.PythonInterpreter;

public class TreeFactory {
	
	 private PythonInterpreter interpreter;
	 private PyObject jyTreeClass;
	/**
	 * Java factory that makes conversions of the python classes in the module
	 * from PyObjects to Java objects
	 */
	
	public TreeFactory() {
		
		if(interpreter == null)
		{
			PythonInterpreter.initialize(System.getProperties(), null, new String[] {});
	        interpreter = new PythonInterpreter(new PyStringMap(), new PySystemState());
		}
		initializeInterpreter();
    }
	
	private void initializeInterpreter(){
		interpreter.exec("from generateTree import RandomTree");
        jyTreeClass = interpreter.get("RandomTree");
	}
	
    /**
     * Method that calls the python module to create a random tree based on the 
     * tree parameters. It handles the conversion from PyObject to Java object.
     * @param parameters
     * @param noActivities
     * @return
     */
	
    public randomTree createTree(String parameters) {
        PyObject treeObj = jyTreeClass.__call__(new PyString(parameters));
        return (randomTree)treeObj.__tojava__(randomTree.class);
    }
    
    public void cleanupInterpreter(){
    	interpreter.cleanup();
    }

    
   

}
