package org.processmining.ptandloggenerator.algorithms;

import org.processmining.ptandloggenerator.interfaces.uniqueTraces;
import org.python.core.PyInteger;
import org.python.core.PyObject;
import org.python.core.PyString;
import org.python.core.PyStringMap;
import org.python.core.PySystemState;
import org.python.util.PythonInterpreter;

public class UniqueTracesFactory {

	 private PythonInterpreter interpreter;
	 private PyObject jyUniqueTracesClass;
	 
	 /**
	 * Java factory that makes conversions of the python classes in the module
	 * from PyObjects to Java objects
	 */
	
	 public UniqueTracesFactory() {
		
		 if(interpreter == null)
		 {
			 PythonInterpreter.initialize(System.getProperties(), null, new String[] {});
			 interpreter = new PythonInterpreter(new PyStringMap(), new PySystemState());
		 }
		 initializeInterpreter();
	 }
	 
	 private void initializeInterpreter(){
		 interpreter.exec("from noUniqueTraces import NoUniqueTraces");
		 jyUniqueTracesClass = interpreter.get("NoUniqueTraces");
	 }
	 
	 /**
	  * Calculate the number of unique traces in a given process tree
	  * @param newickTree: string representation of newick tree
	  * @param loopIterations: maximum times loop is executed
	  * @return double number of unique traces in the process tree
	  */
	 
	 public uniqueTraces calculateNoUniqueTraces(String newickTree, int loopIterations) {
		 PyObject uniqueTracesObj = jyUniqueTracesClass.__call__(new PyString(newickTree),
	        										new PyInteger(loopIterations));
		 return (uniqueTraces)uniqueTracesObj.__tojava__(uniqueTraces.class);
	 }
	 
	 public void cleanupInterpreter(){
		 interpreter.cleanup();
	 }
	 
}
