package org.processmining.ptandloggenerator.dialogs;

import java.math.BigDecimal;
import java.math.RoundingMode;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import com.fluxicon.slickerbox.colors.SlickerColors;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;

public class dataEvaluationSettings {
	int maxNoCases;
	int minNoCases;
	int maxInput;
	int maxCutoff;
	boolean useOverlapping;
	
	BigDecimal targetDeterminism;
	
	/** Spinner for the maximum number of cases to generate */
	JSpinner spinMaxNoCases;
	/** Spinner for the minimum number of cases to generate */
	JSpinner spinMinNoCases;
	/** Spinner for the maximum number of case attribute linked to a routing decision */
	JSpinner spinMaxInput;
	/** Spinner for the cutoffs + 1 = maximum number of intervals for numerical case attributes */
	JSpinner spinMaxCutoff;
	/** Spinner for the target determinism of the routing decisions */
	JSpinner spinTargetDeterminism;
	/** Combobox for the selection of decision mining algorithm */
	@SuppressWarnings("rawtypes")
	JComboBox miner;
	
	public JComponent initComponents() {
		
		//Factory to create ProM swing components
		SlickerFactory factory = SlickerFactory.instance();

		//Setting the Panel
		JPanel panel = factory.createRoundedPanel(20, SlickerColors.COLOR_BG_1);
		panel.setLayout(new TableLayout(new double[][] { { 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50 },
				{ 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30 } }));
		
		//MIN NO CASES SPINNER-------------------------------------------
		int currentCases = 200;
		int minCases = new Integer(1);
		int maxCases = new Integer(Integer.MAX_VALUE);
		int stepCases = new Integer(20);
		//creating the number spinner model
		SpinnerNumberModel modelMinNoCases = new SpinnerNumberModel(currentCases,
				minCases, maxCases, stepCases);
		//creating the number spinner
		spinMinNoCases = new JSpinner(modelMinNoCases);
		JFormattedTextField tfMinNoCases = ((JSpinner.DefaultEditor) spinMinNoCases.getEditor()).getTextField();
		tfMinNoCases.setBackground(SlickerColors.COLOR_BG_3);
		tfMinNoCases.setForeground(SlickerColors.COLOR_FG);
		spinMinNoCases.setToolTipText("Specify the minimum number of cases in the log");
		//Add
		panel.add(factory.createLabel("<html><h2>The minimum and maximum number of cases in the log</h2></html>"), "0, 0, 10, 0");
		panel.add(factory.createLabel("The simulator will produce one event log with 10-fold cross validation for each input model!"), "0, 1, 10, 1");
		panel.add(spinMinNoCases, "0, 2, 1, 2");
		panel.add(factory.createLabel("  Minimum number of cases"), "2, 2, 6, 2");
		
		//MAX NO CASES SPINNER-------------------------------------------
		int currentMaxCases = 1000;
		//creating the number spinner model
		SpinnerNumberModel modelMaxNoCases = new SpinnerNumberModel(currentMaxCases,
				minCases, maxCases, stepCases);
		//creating the number spinner
		spinMaxNoCases = new JSpinner(modelMaxNoCases);
		JFormattedTextField tfMaxNoCases = ((JSpinner.DefaultEditor) spinMaxNoCases.getEditor()).getTextField();
		tfMaxNoCases.setBackground(SlickerColors.COLOR_BG_3);
		tfMaxNoCases.setForeground(SlickerColors.COLOR_FG);
		spinMaxNoCases.setToolTipText("Specify the maximum number of cases in the log");
		//Add
		panel.add(spinMaxNoCases, "7, 2, 8, 2");
		panel.add(factory.createLabel("  Maximum number of cases"), "9, 2, 15, 2");
		
		//NO MAX INPUT SPINNER-------------------------------------------
		int currentMax = 3;
		int minMax = new Integer(1);
		int maxMax = new Integer(10);
		int stepMax = new Integer(1);
		//creating the number spinner model
		SpinnerNumberModel modelMaxInput = new SpinnerNumberModel(currentMax,
				minMax, maxMax, stepMax);
		//creating the number spinner
		spinMaxInput = new JSpinner(modelMaxInput);
		JFormattedTextField tfMaxInput = ((JSpinner.DefaultEditor) spinMaxInput.getEditor()).getTextField();
		tfMaxInput.setBackground(SlickerColors.COLOR_BG_3);
		tfMaxInput.setForeground(SlickerColors.COLOR_FG);
		spinMaxInput.setToolTipText("Specify the maximum number of case attributes per routing decision");
		//Add
		panel.add(factory.createLabel("<html><h2>Case attribute specifications</h2></html>"), "0, 4, 10, 4");
		panel.add(spinMaxInput, "0, 5, 1, 5");
		panel.add(factory.createLabel("  Maximum number of case attributes per decision"), "2, 5, 10, 5");
		
		//NO MAX CUTOFF SPINNER-------------------------------------------
		//creating the number spinner model
		SpinnerNumberModel modelMaxCutoff = new SpinnerNumberModel(currentMax,
				minMax, maxMax, stepMax);
		//creating the number spinner
		spinMaxCutoff = new JSpinner(modelMaxCutoff);
		JFormattedTextField tfMaxCutoff = ((JSpinner.DefaultEditor) spinMaxCutoff.getEditor()).getTextField();
		tfMaxCutoff.setBackground(SlickerColors.COLOR_BG_3);
		tfMaxCutoff.setForeground(SlickerColors.COLOR_FG);
		spinMaxCutoff.setToolTipText("Specify the maximum number of cutoffs for numerical case attributes");
		//Add
		panel.add(spinMaxCutoff, "0, 7, 1, 7");
		panel.add(factory.createLabel("  Maximum number of cutoffs for numerical case attributes"), "2, 7, 10, 7");
		
		//TARGET DETERMINISM SPINNER-------------------------------------------
		double defaultTargetDeterminism = 0.5;
		double minTargetDeterminism = 0.0;
		double maxTargetDeterminism = 1.0;
		double stepTargetDeterminism = 0.05;
		//creating the number spinner model
		SpinnerNumberModel modelTargetDeterminism = new SpinnerNumberModel(defaultTargetDeterminism,
				minTargetDeterminism, maxTargetDeterminism, stepTargetDeterminism);
		//creating the number spinner
		spinTargetDeterminism = new JSpinner(modelTargetDeterminism);
		JFormattedTextField tfTargetDeterminism = ((JSpinner.DefaultEditor) spinTargetDeterminism.getEditor()).getTextField();
		tfTargetDeterminism.setBackground(SlickerColors.COLOR_BG_3);
		tfTargetDeterminism.setForeground(SlickerColors.COLOR_FG);
		spinTargetDeterminism.setToolTipText("Specify the target determinism level");
		//Add
		panel.add(factory.createLabel("<html><h2>The determinism level of routing decisions</h2></html>"), "0, 9, 10, 9");
		panel.add(factory.createLabel("0 means non-deterministic, 1 fully-deterministic routing decisions"), "0, 10, 10, 10");
		panel.add(spinTargetDeterminism, "0, 11, 1, 11");
		panel.add(factory.createLabel("  Target determinism level of routing decisions"), "2, 11, 10, 11");
		
		//DECISION MINING ALGORITHM
        miner = SlickerFactory.instance().createComboBox(new String[] {"Overlapping rules", 
        		"Decision tree with alignment"});
        panel.add(factory.createLabel("<html><h2>Decision mining settings</h2></html>"), "0, 13, 10, 13");
        panel.add(miner, "0, 14, 2, 14");
        panel.add(factory.createLabel("  Choose decision mining algorithm"), "3, 14, 10, 14");
		
		return panel;
	}
			
	public void setMinNoCases(){
		minNoCases = (int) spinMinNoCases.getValue();
	}
	
	public int getMinNoCases(){
		return minNoCases;
	}
	
	public void setMaxNoCases(){
		maxNoCases = (int) spinMaxNoCases.getValue();
	}
	
	public int getMaxNoCases(){
		return maxNoCases;
	}
	
	public void setMaxInput() {
		maxInput = (int) spinMaxInput.getValue();
	}
	
	public String getMaxInput() {
		return Integer.toString(maxInput);
	}
	
	public void setMaxCutoff() {
		maxCutoff = (int) spinMaxCutoff.getValue();
	}
	
	public String getMaxCutoff() {
		return Integer.toString(maxCutoff);
	}
	
	public void setTargetDeterminism(){
		targetDeterminism = new BigDecimal((double) spinTargetDeterminism.getValue()).setScale(2, RoundingMode.HALF_UP);
	}
	
	public String getTargetDeterminism(){
		return targetDeterminism.toString();
	}
	
	public void setMiner() {
		if (miner.getSelectedIndex() == 0)
			useOverlapping = true;
		else
			useOverlapping = false;
	}
	
	public boolean getMiner() {
		return useOverlapping;
	}
}
