package org.processmining.ptandloggenerator.dialogs;

import java.math.BigDecimal;
import java.math.RoundingMode;

import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;

import com.fluxicon.slickerbox.colors.SlickerColors;
import com.fluxicon.slickerbox.factory.SlickerFactory;

import info.clearthought.layout.TableLayout;

public class dataLogSettings {
	int noCases;
	int noNoisyCases;
	int maxInput;
	int maxCutoff;
	BigDecimal targetDeterminism;
	
	/** Spinner for the number of cases to generate */
	JSpinner spinNoCases;
	/** Spinner for the number of noisy cases to generate */
	JSpinner spinNoNoisyCases;
	/** Spinner for the maximum number of case attribute linked to a routing decision */
	JSpinner spinMaxInput;
	/** Spinner for the cutoffs + 1 = maximum number of intervals for numerical case attributes */
	JSpinner spinMaxCutoff;
	/** Spinner for the target determinism of the routing decisions */
	JSpinner spinTargetDeterminism;
	
	public JComponent initComponents() {
		//Factory to create ProM swing components
		SlickerFactory factory = SlickerFactory.instance();

		//Setting the Panel
		JPanel panel = factory.createRoundedPanel(20, SlickerColors.COLOR_BG_1);
		panel.setLayout(new TableLayout(new double[][] { { 50, 50, 50, 50, 50, 50, 50, 50, 50, 50, 50 },
				{ 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30, 30 } }));
		
		//NO CASES SPINNER-------------------------------------------
		int currentNoCases = 100;
		int minNoCases = new Integer(1);
		int maxNoCases = new Integer(Integer.MAX_VALUE);
		int stepNoCases = new Integer(1);
		//creating the number spinner model
		SpinnerNumberModel modelNoCases = new SpinnerNumberModel(currentNoCases,
				minNoCases, maxNoCases, stepNoCases);
		//creating the number spinner
		spinNoCases = new JSpinner(modelNoCases);
		JFormattedTextField tfNoCases = ((JSpinner.DefaultEditor) spinNoCases.getEditor()).getTextField();
		tfNoCases.setBackground(SlickerColors.COLOR_BG_3);
		tfNoCases.setForeground(SlickerColors.COLOR_FG);
		spinNoCases.setToolTipText("Specify the number of cases in the training log");
		//Add
		panel.add(factory.createLabel("<html><h2>The number of cases in the training and test log</h2></html>"), "0, 0, 10, 0");
		panel.add(factory.createLabel("The simulator will produce one event log with training and test cases for each input model!"), "0, 1, 10, 1");
		panel.add(spinNoCases, "0, 2, 1, 2");
		panel.add(factory.createLabel("  Number of training cases"), "2, 2, 4, 2");
		
		//NO NOISY CASES SPINNER-------------------------------------------
		int currentNoNoisyCases = 10;
		//creating the number spinner model
		SpinnerNumberModel modelNoNoisyCases = new SpinnerNumberModel(currentNoNoisyCases,
				minNoCases, maxNoCases, stepNoCases);
		//creating the number spinner
		spinNoNoisyCases = new JSpinner(modelNoNoisyCases);
		JFormattedTextField tfNoNoisyCases = ((JSpinner.DefaultEditor) spinNoNoisyCases.getEditor()).getTextField();
		tfNoNoisyCases.setBackground(SlickerColors.COLOR_BG_3);
		tfNoNoisyCases.setForeground(SlickerColors.COLOR_FG);
		spinNoNoisyCases.setToolTipText("Specify the number of noisy cases in the test log (that contains equal number of fitting cases)");
		//Add
		panel.add(spinNoNoisyCases, "0, 4, 1, 4");
		panel.add(factory.createLabel("  Number of noisy test cases"), "2, 4, 4, 4");
		
		//NO MAX INPUT SPINNER-------------------------------------------
		int currentMax = 3;
		int minMax = new Integer(1);
		int maxMax = new Integer(10);
		int stepMax = new Integer(1);
		//creating the number spinner model
		SpinnerNumberModel modelMaxInput = new SpinnerNumberModel(currentMax,
				minMax, maxMax, stepMax);
		//creating the number spinner
		spinMaxInput = new JSpinner(modelMaxInput);
		JFormattedTextField tfMaxInput = ((JSpinner.DefaultEditor) spinMaxInput.getEditor()).getTextField();
		tfMaxInput.setBackground(SlickerColors.COLOR_BG_3);
		tfMaxInput.setForeground(SlickerColors.COLOR_FG);
		spinMaxInput.setToolTipText("Specify the maximum number of case attributes per routing decision");
		//Add
		panel.add(factory.createLabel("<html><h2>Case attribute specifications</h2></html>"), "0, 6, 10, 6");
		panel.add(spinMaxInput, "0, 8, 1, 8");
		panel.add(factory.createLabel("  Maximum number of case attributes per decision"), "2, 8, 10, 8");
		
		//NO MAX CUTOFF SPINNER-------------------------------------------
		//creating the number spinner model
		SpinnerNumberModel modelMaxCutoff = new SpinnerNumberModel(currentMax,
				minMax, maxMax, stepMax);
		//creating the number spinner
		spinMaxCutoff = new JSpinner(modelMaxCutoff);
		JFormattedTextField tfMaxCutoff = ((JSpinner.DefaultEditor) spinMaxCutoff.getEditor()).getTextField();
		tfMaxCutoff.setBackground(SlickerColors.COLOR_BG_3);
		tfMaxCutoff.setForeground(SlickerColors.COLOR_FG);
		spinMaxCutoff.setToolTipText("Specify the maximum number of cutoffs for numerical case attributes");
		//Add
		panel.add(spinMaxCutoff, "0, 10, 1, 10");
		panel.add(factory.createLabel("  Maximum number of cutoffs for numerical case attributes"), "2, 10, 10, 10");
		
		//TARGET DETERMINISM SPINNER-------------------------------------------
		double defaultTargetDeterminism = 0.5;
		double minTargetDeterminism = 0.0;
		double maxTargetDeterminism = 1.0;
		double stepTargetDeterminism = 0.05;
		//creating the number spinner model
		SpinnerNumberModel modelTargetDeterminism = new SpinnerNumberModel(defaultTargetDeterminism,
				minTargetDeterminism, maxTargetDeterminism, stepTargetDeterminism);
		//creating the number spinner
		spinTargetDeterminism = new JSpinner(modelTargetDeterminism);
		JFormattedTextField tfTargetDeterminism = ((JSpinner.DefaultEditor) spinTargetDeterminism.getEditor()).getTextField();
		tfTargetDeterminism.setBackground(SlickerColors.COLOR_BG_3);
		tfTargetDeterminism.setForeground(SlickerColors.COLOR_FG);
		spinTargetDeterminism.setToolTipText("Specify the target determinism level");
		//Add
		panel.add(factory.createLabel("<html><h2>The determinism level of routing decisions</h2></html>"), "0, 12, 10, 12");
		panel.add(factory.createLabel("0 means non-deterministic, 1 fully-deterministic routing decisions"), "0, 13, 10, 13");
		panel.add(spinTargetDeterminism, "0, 14, 1, 14");
		panel.add(factory.createLabel("  Target determinism level of routing decisions"), "2, 14, 10, 14");
		
		return panel;
	}
	
	public void setNoCases(){
		noCases = (int) spinNoCases.getValue();
	}
	
	public String getNoCases(){
		return Integer.toString(noCases);
	}
	
	public void setNoNoisyCases(){
		noNoisyCases = (int) spinNoNoisyCases.getValue();
	}
	
	public String getNoNoisyCases(){
		return Integer.toString(noNoisyCases);
	}
	
	public void setMaxInput() {
		maxInput = (int) spinMaxInput.getValue();
	}
	
	public String getMaxInput() {
		return Integer.toString(maxInput);
	}
	
	public void setMaxCutoff() {
		maxCutoff = (int) spinMaxCutoff.getValue();
	}
	
	public String getMaxCutoff() {
		return Integer.toString(maxCutoff);
	}
	
	public void setTargetDeterminism(){
		targetDeterminism = new BigDecimal((double) spinTargetDeterminism.getValue()).setScale(2, RoundingMode.HALF_UP);
	}
	
	public String getTargetDeterminism(){
		return targetDeterminism.toString();
	}
}
