package org.processmining.ptandloggenerator.models;

import java.util.Calendar;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.extension.std.XLifecycleExtension;
import org.deckfour.xes.extension.std.XTimeExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryBufferedImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.python.core.PyDictionary;
import org.python.core.PyFloat;
import org.python.core.PyList;
import org.python.core.PyObject;

public class Case {
	
	private XTrace trace;
	private Calendar calendar = Calendar.getInstance();

	/**
	 * Object represents a trace as a list of events with activity names and data attributes
	 * 
	 * @param trace_input
	 * @param attributes: {key:value}
	 */

	public Case(PyList trace_input, PyDictionary attributes, int index) {

		XFactory factory = new XFactoryBufferedImpl();

		XAttributeMap traceMap = factory.createAttributeMap();
		XAttribute id = factory.createAttributeLiteral("concept:name", Integer.toString(index+1),
				XConceptExtension.instance());
		traceMap.put(id.getKey(), id);

		trace = factory.createTrace(traceMap);

		int count = trace_input.__len__();

		for (int i = 0; i < count; i++) {
			PyObject act = trace_input.__getitem__(i);
			String newact = act.asString();

			XAttributeMap attMap = factory.createAttributeMap();
			XAttribute nameAttribute = factory.createAttributeLiteral("concept:name", newact,
					XConceptExtension.instance());
			attMap.put(nameAttribute.getKey(), nameAttribute);
			// go through all attributes and add to attMap
			addToAttributeMap(attributes,attMap,factory);
			// add date attribute
			calendar.add(Calendar.SECOND, 1);
			XAttribute timeAttribute = factory.createAttributeTimestamp("time:timestamp", calendar.getTime(),
					XTimeExtension.instance());
			attMap.put(timeAttribute.getKey(), timeAttribute);
			// make event instance
			XEvent event = factory.createEvent(attMap);
			// add lifecycle transition
			XLifecycleExtension.instance().assignStandardTransition(event,XLifecycleExtension.StandardModel.COMPLETE);
			trace.add(event);
		}
	}

	public XTrace getCase() {
		return trace;
	}
	
	public void addToAttributeMap(PyDictionary pyDict, XAttributeMap map, XFactory factory) {
		PyList keys = pyDict.keys();
		int count = keys.__len__();		
		PyObject pyKey, pyValue;
		String javaKey, javaValue;
		for (int i = 0; i < count; ++i){
			pyKey = keys.__getitem__(i);
			javaKey = pyKey.asString();
			pyValue = pyDict.__getitem__(pyKey);
			javaValue = pyValue.toString();
			XAttribute attr;
			if (pyValue.getClass() == PyFloat.class) {
				attr = factory.createAttributeContinuous(javaKey, Double.parseDouble(javaValue),
						XConceptExtension.instance());
			}
			else {
				attr = factory.createAttributeLiteral(javaKey, javaValue,
						XConceptExtension.instance());
			}
			map.put(attr.getKey(), attr);
		}
	}
	
	public int compareTo(Case o) {
		XTrace newTrace = o.getCase();
		
		if(newTrace.size() != trace.size())
			return 1;
		else{
			//check trace attributes
			
			//check events
			for(int i = 0 ; i < trace.size() ; i++){
				XAttributeMap newMap = newTrace.get(i).getAttributes();
				XAttributeMap map = trace.get(i).getAttributes();
				
				for(XAttribute attribute : map.values()){
					if(!newMap.containsKey(attribute.getKey()))
						return 1;
					else{
						if(newMap.get(attribute.getKey()).compareTo(map.get(attribute.getKey())) != 0)
							return 1;
					}
				}
			}
		}		
		return 0;
	}

}
