package org.processmining.ptandloggenerator.parameters;

/**
 * This parameter object contains the probabilites of each type of noise to be
 * applied (independently) to a trace.
 * 
 * @author abolt
 *
 */
public class NoiseGeneratorSettings {

	double probAddEvent, probRemoveEvent, probDuplicateEvent, probSwapConsecutiveEvents, probSwapRandomEvents,
			probRemoveHead, probRemoveBody, probRemoveTail;

	//this variable will define if a single noise operation is enforced (chosen randomly from all non-0 probabilities, regardless of the values)
	private boolean forceSingleNoiseOp;

	public NoiseGeneratorSettings() {
	}

	public static NoiseGeneratorSettings getDefaultSettings() {
		NoiseGeneratorSettings settings = new NoiseGeneratorSettings();
		settings.setToLocalNoiseDefaults();
		return settings;
	}
	
	public static NoiseGeneratorSettings getIntegratedDefaultSettings() {
		NoiseGeneratorSettings settings = new NoiseGeneratorSettings();
		settings.setToIntegratedDefaults();
		return settings;
	}

	public void setToDefaults() {
		setProbAddEvent(0.1);
		setProbRemoveEvent(0.1);
		setProbDuplicateEvent(0.1);
		setProbSwapConsecutiveEvents(0.1);
		setProbSwapRandomEvents(0.1);
		setProbRemoveHead(0.1);
		setProbRemoveBody(0.1);
		setProbRemoveTail(0.1);
		setForceSingleNoiseOp(false);
	}

	/**
	 * This is used for inducing local noise to traces, this may result in
	 * not-so-obvious noise that could difficult a discovery algorithm's job
	 */
	public void setToLocalNoiseDefaults() {
		setProbAddEvent(0);
		setProbRemoveEvent(0.1);
		setProbDuplicateEvent(0.1);
		setProbSwapConsecutiveEvents(0.1);
		setProbSwapRandomEvents(0);
		setProbRemoveHead(0);
		setProbRemoveBody(0);
		setProbRemoveTail(0);
		setForceSingleNoiseOp(true);
	}
	
	/**
	 * This method is used for the plugin that integrates noise generation and log generation.
	 * Each probability is set equal.
	 */
	public void setToIntegratedDefaults() {
		setProbAddEvent(0);
		setProbRemoveEvent(0.2);
		setProbDuplicateEvent(0);
		setProbSwapConsecutiveEvents(0.1);
		setProbSwapRandomEvents(0.1);
		setProbRemoveHead(0.2);
		setProbRemoveBody(0.2);
		setProbRemoveTail(0.2);
		setForceSingleNoiseOp(true);
	}

	public double getProbAddEvent() {
		return probAddEvent;
	}

	public void setProbAddEvent(double probAddEvent) {
		this.probAddEvent = probAddEvent;
	}

	public double getProbRemoveEvent() {
		return probRemoveEvent;
	}

	public void setProbRemoveEvent(double probRemoveEvent) {
		this.probRemoveEvent = probRemoveEvent;
	}

	public double getProbDuplicateEvent() {
		return probDuplicateEvent;
	}

	public void setProbDuplicateEvent(double probDuplicateEvent) {
		this.probDuplicateEvent = probDuplicateEvent;
	}

	public double getProbSwapConsecutiveEvents() {
		return probSwapConsecutiveEvents;
	}

	public void setProbSwapConsecutiveEvents(double probSwapConsecutiveEvents) {
		this.probSwapConsecutiveEvents = probSwapConsecutiveEvents;
	}

	public double getProbSwapRandomEvents() {
		return probSwapRandomEvents;
	}

	public void setProbSwapRandomEvents(double probSwapRandomEvents) {
		this.probSwapRandomEvents = probSwapRandomEvents;
	}

	public double getProbRemoveHead() {
		return probRemoveHead;
	}

	public void setProbRemoveHead(double probRemoveHead) {
		this.probRemoveHead = probRemoveHead;
	}

	public double getProbRemoveBody() {
		return probRemoveBody;
	}

	public void setProbRemoveBody(double probRemoveBody) {
		this.probRemoveBody = probRemoveBody;
	}

	public double getProbRemoveTail() {
		return probRemoveTail;
	}

	public void setProbRemoveTail(double probRemoveTail) {
		this.probRemoveTail = probRemoveTail;
	}

	public boolean getForceSingleNoiseOp() {
		return forceSingleNoiseOp;
	}

	public void setForceSingleNoiseOp(boolean forceSingleNoiseOp) {
		this.forceSingleNoiseOp = forceSingleNoiseOp;
	}

}
