package org.processmining.ptandloggenerator.parameters;

public class TreeParameters {

	/**
	 * parameters used for generating a tree
	 */

	private int noActivities;

	private String patternProbabilities;

	private int noTrees;

	private double mode, min, max;

	/**
	 * The constructor takes a string as input and then sets all the parameters
	 * needed as input for generating a process tree
	 * 
	 * @param parameterLine
	 */

	public TreeParameters(String parameterLine) {
		String separator = ";";
		String[] csv_parameters;
		csv_parameters = parameterLine.split(separator);
		mode = Double.parseDouble(csv_parameters[0]);
		min = Double.parseDouble(csv_parameters[1]);
		max = Double.parseDouble(csv_parameters[2]);

		//setNoActivities();
		setPatternProbabilities(parameterLine);
		setNoTrees(Integer.parseInt(csv_parameters[12]));
	}

	/**
	 * sets the pattern probabilities that are passed to the python module
	 * 
	 * @param probabilities
	 */

	public void setPatternProbabilities(String probabilities) {
		patternProbabilities = probabilities;
	}

	/**
	 * 
	 * @return the pattern probabilities represented as a string
	 */

	public String getPatternProbabilities() {
		return patternProbabilities;
	}

	/**
	 * set the number of trees that need to be generated
	 * 
	 * @param noModels
	 */

	public void setNoTrees(int noModels) {
		noTrees = noModels;
	}

	/**
	 * 
	 * @return the number of trees that need to be generated
	 */

	public int getNoTrees() {
		return noTrees;
	}
	

	/**
	 * The method sets the number of activities in the final process tree to be
	 * generated by drawing a random number from a triangular distribution
	 * 
	 * @param mode
	 *            the mode or most frequent value of the distribution
	 * @param min
	 *            the minimum value of the distribution
	 * @param max
	 *            the maximum value of the distribution
	 */
	@Deprecated
	public void setNoActivities() {
		noActivities = (int) Math.round(triangularDistribution(min, max, mode));
	}

	/**
	 * 
	 * @return the number of activities in the final process tree
	 */
	@Deprecated
	public int getNoActivities() {
		setNoActivities();
		return noActivities;
	}

	/**
	 * @Deprecated Method that implements a draw from a triangular distribution, now in python code!
	 * 
	 * @param a
	 *            min
	 * @param b
	 *            max
	 * @param c
	 *            mode
	 * @return a random value from the distribution
	 */
	@Deprecated
	public static double triangularDistribution(double a, double b, double c) {
		double F = (c - a) / (b - a);
		double rand = Math.random();
		if (rand < F) {
			return a + Math.sqrt(rand * (b - a) * (c - a));
		} else {
			return b - Math.sqrt((1 - rand) * (b - a) * (b - c));
		}
	}

}
