package org.processmining.ptandloggenerator.parsers;

import java.util.EmptyStackException;
import java.util.Stack;

import org.processmining.ptandloggenerator.models.JavaTree;
import org.processmining.ptandloggenerator.models.JavaTreeNode;


public class NewickTreeParser {
	
	private String newickTree;
	
	private JavaTreeNode rootNode;
	
	private JavaTree t;
	
	public NewickTreeParser(String newickTree){
		this.newickTree = newickTree;
		
		parseString();
	}
	
	public void parseString(){
		final char openBracket = '(', closeBracket = ')', childSeparator = ',',
	        	treeTerminator = ';', infoSeparator = ':';
		
		String value = "";
        rootNode = new JavaTreeNode();
        t = new JavaTree();
        t.setRootNode(rootNode);
        Stack<JavaTreeNode> nodeStack = new Stack<JavaTreeNode>();
        nodeStack.push(rootNode);
        JavaTreeNode lastNamed = null;
        boolean nameNext = true;
        double probability;
		
		for (int i = 0; i < newickTree.length(); i++){
		    char c = newickTree.charAt(i);        
		    
	    	if (c == openBracket){
	    		if (value != ""){
	    			try {
	    				probability = Double.parseDouble(value);
	    				if (nameNext)
	            		    lastNamed = popAndName(value, nodeStack);
	            		else
	            		{
	            		    if (lastNamed != null)
	            		        lastNamed.setWeight(probability);
	            		    else
	            		        System.err.println("Error: can't set value " + value + " to a null node");
	            		    lastNamed = null;
	            		}
	            		nameNext = false;
	    			} catch (NumberFormatException e){
	    				if (!nameNext)
	            	        System.err.println("Error: didn't expect this name here: " + value);
	            	    lastNamed = popAndName(value, nodeStack);
	            		nameNext = false;
	    			}
	    			value = "";
	    		}
	    		nodeStack.push(new JavaTreeNode());
        	    nameNext = true;
	    	}
	    	
	    	else if (c == closeBracket){
	    		if (value != ""){
	    			try {
	    				probability = Double.parseDouble(value);
	    				if (nameNext)
	            		    lastNamed = popAndName(value, nodeStack);
	            		else
	            		{
	            		    if (lastNamed != null)
	            		        lastNamed.setWeight(probability);
	            		    else
	            		        System.err.println("Error: can't set value " + value + " to a null node");
	            		    lastNamed = null;
	            		}
	            		nameNext = false;
	    			} catch (NumberFormatException e){
	    				if (!nameNext)
	            	        System.err.println("Error: didn't expect this name here: " + value);
	            	    lastNamed = popAndName(value, nodeStack);
	            		nameNext = false;
	    			}
	    			value = "";
	    		}
	    		if (nameNext)
        	        lastNamed = popAndName(null, nodeStack);
        	    nameNext = true;
	    	}
	    	
	    	else if (c == childSeparator){
	    		if (value != ""){
	    			try {
	    				probability = Double.parseDouble(value);
	    				if (nameNext)
	            		    lastNamed = popAndName(value, nodeStack);
	            		else
	            		{
	            		    if (lastNamed != null)
	            		        lastNamed.setWeight(probability);
	            		    else
	            		        System.err.println("Error: can't set value " + value + " to a null node");
	            		    lastNamed = null;
	            		}
	            		nameNext = false;
	    			} catch (NumberFormatException e){
	    				if (!nameNext)
	            	        System.err.println("Error: didn't expect this name here: " + value);
	            	    lastNamed = popAndName(value, nodeStack);
	            		nameNext = false;
	    			}
	    			value = "";
	    		}
	    		if (nameNext)
        	        lastNamed = popAndName(null, nodeStack);
        	    nodeStack.push(new JavaTreeNode());
        	    nameNext = true;
	    	}
	    	
	    	else if (c == treeTerminator){
	    		if (value != ""){
	    			try {
	    				probability = Double.parseDouble(value);
	    				if (nameNext)
	            		    lastNamed = popAndName(value, nodeStack);
	            		else
	            		{
	            		    if (lastNamed != null)
	            		        lastNamed.setWeight(probability);
	            		    else
	            		        System.err.println("Error: can't set value " + value + " to a null node");
	            		    lastNamed = null;
	            		}
	            		nameNext = false;
	    			} catch (NumberFormatException e){
	    				if (!nameNext)
	            	        System.err.println("Error: didn't expect this name here: " + value);
	            	    lastNamed = popAndName(value, nodeStack);
	            		nameNext = false;
	    			}
	    			value = "";
	    		}
	    		if (nameNext)
        	        lastNamed = popAndName(null, nodeStack);
        	    nameNext = false;
	    	}
	    	
	    	else if (c == infoSeparator){
	    		if (value != ""){
	    			try {
	    				probability = Double.parseDouble(value);
	    				if (nameNext)
	            		    lastNamed = popAndName(value, nodeStack);
	            		else
	            		{
	            		    if (lastNamed != null)
	            		        lastNamed.setWeight(probability);
	            		    else
	            		        System.err.println("Error: can't set value " + value + " to a null node");
	            		    lastNamed = null;
	            		}
	            		nameNext = false;
	    			} catch (NumberFormatException e){
	    				if (!nameNext)
	            	        System.err.println("Error: didn't expect this name here: " + value);
	            	    lastNamed = popAndName(value, nodeStack);
	            		nameNext = false;
	    			}
	    			value = "";
	    		}
	    		if (nameNext)
        	        lastNamed = popAndName(null, nodeStack);
        	    nameNext = false;
	    	}
	    	
	    	else
	    		value += String.valueOf(c); 	
		    	
		}
		if (!nodeStack.isEmpty())
            System.err.println("Node stack still has " + nodeStack.size() + " things");
        t.postProcess();
        
	}
	
	private JavaTreeNode popAndName(String name, Stack<JavaTreeNode> nodeStack)
    {
		JavaTreeNode topNode = nodeStack.pop();
	    if (name == null)
	    {
	    	topNode.label = "";
	    	topNode.setName("");
	    }
	    else
	    {
	    	topNode.label = name;
	    	topNode.setName(name);
	    }
	    try
	    {
	    	JavaTreeNode parent = nodeStack.peek();
	    	parent.addChild(topNode);
	    }
	    catch (EmptyStackException e)
	    {
	        if (topNode != rootNode)
	            System.out.println("Parser error on node " + topNode);
	    }
	    topNode.setExtremeLeaves(); // sets leftmost and rightmost leaf, non-recursive
	    topNode.setNumberLeaves(); // sets number of leaves, non-recursive
	    topNode.linkNodesInPreorder();
	    topNode.linkNodesInPostorder();
	    return topNode;
    }
	
	public JavaTree getTree(){
		return t;
	}
	

}
