package org.processmining.ptandloggenerator.parsers;

import java.util.HashMap;
import java.util.Map;

import org.processmining.processtree.Block;
import org.processmining.processtree.Edge;
import org.processmining.processtree.Expression;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.Task;
import org.processmining.processtree.Variable;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.processtree.impl.AbstractTask;
import org.processmining.processtree.impl.ExpressionImpl;
import org.processmining.processtree.impl.ProcessTreeImpl;
import org.processmining.processtree.impl.VariableImpl;
import org.processmining.ptandloggenerator.models.JavaTree;
import org.processmining.ptandloggenerator.models.JavaTreeNode;
import org.processmining.ptandloggenerator.models.NewickTree;

public class NewickTreeToProcessTree {

	private ProcessTree tree;

	public NewickTreeToProcessTree(NewickTree treeString) {
		
		tree = new ProcessTreeImpl("Tree");
		NewickTreeParser parser = new NewickTreeParser(treeString.getTree());
		JavaTree javaNewick = parser.getTree();
		String nodeName;

		/**
		 * mapping: mapping between java tree nodes and process tree nodes
		 * needed when adding edges
		 */
		Map<JavaTreeNode, Node> mapping = new HashMap<JavaTreeNode, Node>();

		Edge edge;
		Task task;

		for (JavaTreeNode node : javaNewick.nodes) {
			nodeName = node.getName();
			
			if (node.isLeaf()) {
				if (nodeName.equals("tau")) {
					task = new AbstractTask.Automatic("");
					task.setProcessTree(tree);
					tree.addNode(task);
				} else {
					task = new AbstractTask.Manual(nodeName);
					task.setProcessTree(tree);
					tree.addNode(task);
				}

				//add edge to process tree
				JavaTreeNode initParent = node.parent;
				Node parent = mapping.get(initParent);
				edge = ((Block) parent).addChild(task);
				parent.addIncomingEdge(edge);
				tree.addEdge(edge);
				addExpression(parent, task, node, edge);
			}

			else {
				switch (nodeName) {

					case "sequence" :
						Block seq = new AbstractBlock.Seq("Seq");
						seq.setProcessTree(tree);
						tree.addNode(seq);
						mapping.put(node, seq);
						if (node.isRoot())
							tree.setRoot(seq);
						else {
							//add edge to process tree
							JavaTreeNode initParent = node.parent;
							Node parent = mapping.get(initParent);
							edge = ((Block) parent).addChild(seq);
							parent.addIncomingEdge(edge);
							tree.addEdge(edge);
							addExpression(parent, seq, node, edge);
						}
						break;

					case "parallel" :
						Block and = new AbstractBlock.And("And");
						and.setProcessTree(tree);
						tree.addNode(and);
						mapping.put(node, and);
						if (node.isRoot())
							tree.setRoot(and);
						else {
							//add edge to process tree
							JavaTreeNode initParent = node.parent;
							Node parent = mapping.get(initParent);
							edge = ((Block) parent).addChild(and);
							parent.addIncomingEdge(edge);
							tree.addEdge(edge);
							addExpression(parent, and, node, edge);
						}
						break;

					case "choice" :
						Block xor = new AbstractBlock.Xor("xor");
						xor.setProcessTree(tree);
						tree.addNode(xor);
						mapping.put(node, xor);
						if (node.isRoot())
							tree.setRoot(xor);
						else {
							//add edge to process tree
							JavaTreeNode initParent = node.parent;
							Node parent = mapping.get(initParent);
							edge = ((Block) parent).addChild(xor);
							parent.addIncomingEdge(edge);
							tree.addEdge(edge);
							addExpression(parent, xor, node, edge);
						}
						break;

					case "loop" :
						Block loopXor = new AbstractBlock.XorLoop("loop");
						loopXor.setProcessTree(tree);
						tree.addNode(loopXor);
						mapping.put(node, loopXor);
						if (node.isRoot())
							tree.setRoot(loopXor);
						else {
							//add edge to process tree
							JavaTreeNode initParent = node.parent;
							Node parent = mapping.get(initParent);
							edge = ((Block) parent).addChild(loopXor);
							parent.addIncomingEdge(edge);
							tree.addEdge(edge);
							addExpression(parent, loopXor, node, edge);
						}
						break;

					case "or" :
						Block or = new AbstractBlock.Or("or");
						or.setProcessTree(tree);
						tree.addNode(or);
						mapping.put(node, or);
						if (node.isRoot())
							tree.setRoot(or);
						else {
							//add edge to process tree
							JavaTreeNode initParent = node.parent;
							Node parent = mapping.get(initParent);
							edge = ((Block) parent).addChild(or);
							parent.addIncomingEdge(edge);
							tree.addEdge(edge);
							addExpression(parent, or, node, edge);
						}
						break;
				}
			}
		}

	}

	public ProcessTree getProcessTree() {
		return tree;
	}
	
	public void addExpression(Node parent, Node currentNode, JavaTreeNode node, Edge edge){
		if (parent instanceof AbstractBlock.Xor){
			Variable source = new VariableImpl(parent.getName());
			Variable target = new VariableImpl(currentNode.getName());
			Expression expression = new ExpressionImpl(Float.toString(node.getWeight()), source, target);
			tree.addExpression(expression);
			edge.setExpression(expression);
		}
		
	}

}
