package org.processmining.ptandloggenerator.parsers;

import java.util.Collection;

import org.processmining.processtree.Block;
import org.processmining.processtree.Node;
import org.processmining.processtree.ProcessTree;
import org.processmining.processtree.impl.AbstractBlock;
import org.processmining.ptandloggenerator.models.NewickTree;

import com.google.common.collect.Iterables;

public class ProcessTreeToNewickTree {

	//private Map<UUID,Integer> nodeDictionary = new HashMap<UUID,Integer>();
	//private JavaTree javaTree;
	private ProcessTree tree;
	//private int id;
	private String treeString;
	private NewickTree newickTree;
	
	private final String SEQUENCE = "sequence";
	private final String CHOICE = "choice";
	private final String PARALLEL = "parallel";
	private final String LOOP = "loop";
	private final String OR = "or";

	/**
	 * Constructor will transform process tree to a newick tree format. It
	 * traverses through the process tree only once in a preorder format!
	 * 
	 * @param process
	 *            tree: tree
	 */

	public ProcessTreeToNewickTree(ProcessTree tree) {
		this.tree = tree;
		treeString = "";
		convertToNewickString(tree.getRoot(), "1");
		treeString += ";";
		newickTree = new NewickTree(treeString);
	}

	/**
	 * Recursive method to traverse the process tree in preorder. Each node is
	 * added to the newick string
	 * 
	 * @param n
	 *            (node in process tree)
	 * @param weight
	 *            (weight of the branches)
	 */

	public void convertToNewickString(Node n, String weight) {

		if (!n.equals(tree.getRoot())) {
			Collection<Block> parents = n.getParents();
			Block parent = Iterables.get(parents, 0);
			if (parent instanceof AbstractBlock.Xor)
				weight = Double.toString(1.0 / parent.numChildren());
			else
				weight = "1";
		}

		if (n.isLeaf()) {
			if (n.getName().equals(""))
				treeString += "tau";
			else
				treeString += n.getName();
			treeString += ":" + weight;
			return;
		}

		treeString += "(";
		for (int i = 0; i < ((Block) n).numChildren(); i++) {
			convertToNewickString(((Block) n).getChildren().get(i), weight);
			if (i != (((Block) n).numChildren() - 1))
				treeString += ",";
		}
		treeString += ")";
		treeString += translateOperator(n);
		if (tree.getRoot().equals(n))
			weight = "1";
		treeString += ":" + weight;

	}

	public String translateOperator(Node n) {
		
		if (n instanceof AbstractBlock.And)
			return PARALLEL;
		else if (n instanceof AbstractBlock.Seq)
			return SEQUENCE;
		else if (n instanceof AbstractBlock.Xor)
			return CHOICE;
		else if (n instanceof AbstractBlock.XorLoop)
			return LOOP;
		else if (n instanceof AbstractBlock.Or)
			return OR;
		else
			return "";
	}

	public NewickTree getNewickTree() {
		return newickTree;
	}

}
