package org.processmining.ptandloggenerator.plugins;

import java.util.List;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.alphaminer.abstractions.AlphaClassicAbstraction;
import org.processmining.alphaminer.algorithms.AlphaMiner;
import org.processmining.alphaminer.algorithms.AlphaMinerFactory;
import org.processmining.alphaminer.parameters.AlphaMinerParameters;
import org.processmining.alphaminer.parameters.AlphaVersion;
import org.processmining.alphaminer.parameters.DefaultEventClassifier;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.framework.util.Pair;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.petrinets.list.PetriNetList;
import org.processmining.petrinets.list.impl.PetriNetListImpl;
import org.processmining.ptandloggenerator.models.LogCollection;

@Plugin(name = "Apply alpha miner on log collection", parameterLabels = { "Log Collection" }, returnLabels = {
"Petri nets" }, returnTypes = { PetriNetList.class }, 
help = "help")
public class ApplyAlphaLogCollection {
	
	private PetriNetList output;
	
	@UITopiaVariant(affiliation = "Hasselt University", author = "Toon Jouck", email = "toon.jouck@uhasselt.be")
	@PluginVariant(variantLabel = "Mine PN from Log Collection using Alpha Miner", requiredParameterLabels = { 0 })
	public PetriNetList run(PluginContext context, LogCollection logs){
		output = new PetriNetListImpl();
		AlphaMinerParameters parameters = new AlphaMinerParameters();
		parameters.setVersion(AlphaVersion.PLUS_PLUS);
		int i = 1;
		for (XLog log: logs.getXLogCollection()) {
			List<XEventClassifier> classifiers = log.getClassifiers();
			if (classifiers.isEmpty()) {
				classifiers.add(DefaultEventClassifier.get());
			}
			AlphaMiner<XEventClass, ? extends AlphaClassicAbstraction<XEventClass>, ? extends AlphaMinerParameters> miner = AlphaMinerFactory
					.createAlphaMiner(context, log, classifiers.get(0), parameters);
			Pair<Petrinet, Marking> result = miner.run();
			output.add(result.getFirst());
			context.getProvidedObjectManager().createProvidedObject("Alpha++ PN " + Integer.toString(i), result.getFirst(), Petrinet.class , context);
			context.getProvidedObjectManager().createProvidedObject("Marking " + Integer.toString(i), result.getSecond(), Marking.class , context);
			context.getConnectionManager().addConnection(new InitialMarkingConnection(result.getFirst(), result.getSecond()));
			org.processmining.plugins.utils.ProvidedObjectHelper.setFavorite(context, result.getFirst());
			i++;
		}
		return output;
	}

}
