package org.processmining.ptandloggenerator.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.ptandloggenerator.algorithms.UniqueTracesFactory;
import org.processmining.ptandloggenerator.models.NewickTree;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;
import org.processmining.ptandloggenerator.models.NoUniqueTracesModel;

@Plugin(name = "Calculate number of unique traces of a process tree", parameterLabels = { "NewickTreeCollection",
		"Maximum loop iterations" }, returnLabels = { "Number of unique traces" }, returnTypes = {
				NoUniqueTracesModel.class }, help = "help text")
public class CalculateNoUniqueTraces {

	@UITopiaVariant(affiliation = "Hasselt University", author = "Toon Jouck", email = "toon.jouck@uhasselt.be")
	@PluginVariant(variantLabel = "Calculate number of unique traces of a process tree", requiredParameterLabels = {
			0 })
	public NoUniqueTracesModel NoUniqueTracesModel(PluginContext context, NewickTreeCollection trees) {

		NoUniqueTracesModel result = null;
		int loopIterations = 1;

		for (NewickTree tree : trees.getNewickTreeList()) {
			UniqueTracesFactory factory = new UniqueTracesFactory();
			result = new NoUniqueTracesModel(factory.calculateNoUniqueTraces(tree.getTree(), loopIterations));
			factory.cleanupInterpreter();
		}

		return result;
	}

	@PluginVariant(variantLabel = "Calculate number of unique traces of a process tree", requiredParameterLabels = {
			0,1 })
	public NoUniqueTracesModel NoUniqueTracesModel_SingleTree(PluginContext context, NewickTree tree,
			int loopIterations) {

		UniqueTracesFactory factory = new UniqueTracesFactory();
		NoUniqueTracesModel result = new NoUniqueTracesModel(
				factory.calculateNoUniqueTraces(tree.getTree(), loopIterations));
		factory.cleanupInterpreter();

		return result;
	}

}
