package org.processmining.ptandloggenerator.plugins;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.ptandloggenerator.models.EvaluationResults;
import org.processmining.ptandloggenerator.models.EvaluationResultsCollection;

@Plugin(name = "Export Evaluation Results Collection", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Evaluation Results Collection", "File" }, userAccessible = true)
@UIExportPlugin(description = "Evaluation Results Collection", extension = "csv")
public class ExportEvaluationResults {
	@PluginVariant(variantLabel = "Export Evaluation Results Collection", requiredParameterLabels = { 0, 1 })
	public void exportCSV(PluginContext context, File selectedFile,
			EvaluationResultsCollection resultsCollection) {
		BufferedWriter bw = null;
		try {

			bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(selectedFile)));
			bw.write("index,fold,tp,fp,fn,tn,allAlignments,miner");

			for (EvaluationResults results: resultsCollection.getResultsList()) {
				bw.newLine();
				bw.write(Integer.toString(results.getIndex()) + "," +
						Integer.toString(results.getFold()) + "," +
						Integer.toString(results.getTruePositives()) + "," +
						Integer.toString(results.getFalsePositives()) + "," +
						Integer.toString(results.getFalseNegatives()) + "," +
						Integer.toString(results.getTrueNegatives()) + "," +
						Boolean.toString(results.getAllAlignments()) + "," +
						resultsCollection.getTechnique());
			}

		} catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			if (bw != null) {
				try {
					bw.close();
				} catch (Exception exc) {
					// do nothing
				}
			}
		}
	}
}
