package org.processmining.ptandloggenerator.plugins;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.out.XSerializer;
import org.deckfour.xes.out.XesXmlSerializer;
import org.processmining.contexts.uitopia.annotations.UIExportPlugin;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.ptandloggenerator.models.LogCollection;

import com.csvreader.CsvWriter;

@Plugin(name = "ELA export (Event Log Array)", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Event Log Array (XES)", "File" }, userAccessible = true)
@UIExportPlugin(description = "Event Log Array (XES)", extension = "ela")
public class ExportXLogCollection {
	
	@PluginVariant(variantLabel = "ELA export to XES files (Event Log Array)", requiredParameterLabels = { 0, 1 })
	public void exportXLogCollectionGz(PluginContext context, LogCollection collection, File file) throws IOException {
		exportToFile(context, file, new XesXmlSerializer(), collection);
	}
	
	public void exportToFile(PluginContext context, File file, XSerializer logSerializer, LogCollection collection) throws IOException {
		Writer fileWriter = new FileWriter(file);
		CsvWriter csvWriter = new CsvWriter(fileWriter, ',');
		int n = 1;
		for (XLog log: collection.getXLogCollection()) {
			String fileName = file.getName();
			File dir = file.getParentFile();
			String prefix = fileName.substring(0, fileName.indexOf("."));
			File netFile = File.createTempFile(prefix + "_" + n + "_", ".xes", dir);
			csvWriter.write(netFile.getName());
			csvWriter.endRecord();
			System.out.println("Exporting XLog to " + netFile.getName());
			FileOutputStream out = new FileOutputStream(netFile);
			logSerializer.serialize(log, out);
			out.close();
			n++;
		}
		csvWriter.close();
	}

}
