package org.processmining.ptandloggenerator.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.ptandloggenerator.algorithms.LogCollectionGenerator;
import org.processmining.ptandloggenerator.dialogs.logSettings;
import org.processmining.ptandloggenerator.models.LogCollection;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;

@Plugin(name = "Generate Log Collection from Process Trees", parameterLabels = { "Newick Tree Collection", "Log Generator Parameters" }, returnLabels = {
		"Log collection (XES format)" }, returnTypes = { LogCollection.class }, 
		help = "Generate events logs for a given sample of process trees. URI: http://hdl.handle.net/1942/20818")
public class GenerateLogCollection {

	@UITopiaVariant(affiliation = "Hasselt University", author = "Toon Jouck", email = "toon.jouck@uhasselt.be")
	@PluginVariant(variantLabel = "Generate Event Logs (XES) from Process Trees (Newick Trees)", requiredParameterLabels = { 0 })
	public LogCollection run(UIPluginContext context, NewickTreeCollection collection) {
		context.log("Creating log settings");
		//set maximum for progressbar
		context.getProgress().setMaximum(collection.getNewickTreeList().size());
		context.getProgress().setValue(0);
		
		//initialize settings object
		logSettings settings = new logSettings();
		
		//show input wizard
		InteractionResult result = context.showWizard("Log generator Settings", true, true, settings.initComponents());
		
		switch (result) {
			case CANCEL :
				return cancel(context, collection, "The user has cancelled the Log generator!");
			case FINISHED :
				settings.setNoCases();
				context.log("Starting log generation");
				LogCollectionGenerator generator = new LogCollectionGenerator(collection, settings.getNoCases());
				
				int i = 1;
				
				for (XLog log: generator.getLogs().getXLogCollection()){

					//unpack created log
					context.getProvidedObjectManager().createProvidedObject("Log " + Integer.toString(i), log, XLog.class , context);
					i++;
					org.processmining.plugins.utils.ProvidedObjectHelper.setFavorite(context, log);
					
					//advance the progressbar
					context.getProgress().inc();
				}
				return generator.getLogs();
			default :
				System.out.println("problem with the settings");
				return null;
			
		}

	}
	
	@PluginVariant(variantLabel = "Generate Event Logs (XES) from Process Tree (NewickTree), specified parameters", requiredParameterLabels = { 0 })
	public LogCollection run(PluginContext context, NewickTreeCollection collection, int numTrainigTraces) {
		
		LogCollectionGenerator generator = new LogCollectionGenerator(collection, numTrainigTraces);
		
		return generator.getLogs();

	}
	
	
	/**
	* Cancel the future results, display a error message and return all nulls.
	* @param context Context of the plug-in.
	* @param msg Error message.
	* @return Return an object array of nulls;
	*/
	private LogCollection cancel(PluginContext context, NewickTreeCollection collection, String msg) {
		context.log(msg);
		context.getFutureResult(0).cancel(true);
		context.getFutureResult(1).cancel(true);
		return new LogCollectionGenerator(collection, 0).getLogs();
	}

}
