package org.processmining.ptandloggenerator.plugins;

import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.log.csv.CSVFile;
import org.processmining.ptandloggenerator.algorithms.CSVIterator;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;

@Plugin(name = "Generate Process Trees from Population(s)", parameterLabels = { "CSV file with parameters"}, 
returnLabels = { "Newick Tree Collection (Process Trees)" }, returnTypes = { NewickTreeCollection.class }, 
help = "Define a population of process trees from which a random sample of chosen size will be returned. URI: http://hdl.handle.net/1942/20818")
public class GenerateTree {
	
	/*
	 * Plugin that generates a Newick Tree Collection based on population specifications from csv-file
	 */
	
	@UITopiaVariant(affiliation = "Hasselt University", author = "Toon Jouck", email = "toon.jouck@uhasselt.be")
	@PluginVariant(variantLabel = "Generate Process Trees  from a CSV file", requiredParameterLabels = { 0 })
	public NewickTreeCollection run(PluginContext context, CSVFile file) {
		context.log("Creating process trees");
		CSVIterator iterator = new CSVIterator(file.getFile().toString());
		return iterator.getNewickTrees();		
	}

}
