package org.processmining.ptandloggenerator.plugins.data;

import java.util.ArrayList;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.log.models.EventLogArray;
import org.processmining.log.models.impl.EventLogArrayImpl;

@Plugin(name = "Split logs in training/test logs", parameterLabels = { "Event log" }, returnLabels = {
"Event log array" }, returnTypes = { EventLogArray.class }, 
help = "help")
public class DataEvaluation {
	
	@UITopiaVariant(affiliation = "Hasselt University", author = "Toon Jouck", email = "toon.jouck@uhasselt.be")
	@PluginVariant(variantLabel = "Evaluate decision mining algorithm", requiredParameterLabels = { 0 })
	public EventLogArray evaluate(UIPluginContext context, XLog log) throws Exception {
		// set log classifier
		log.getClassifiers().add(XLogInfoImpl.NAME_CLASSIFIER);
		// first split the log in fitting and non-fitting logs
		ArrayList<XLog> splitLogs = splitLog(context,log,2*(log.size()/3));
		
		// create data log collection
		EventLogArray output = new EventLogArrayImpl();
		
		// do 10-fold cross-validation
		for(int fold=1; fold<11; fold++) {
			//System.out.println("Validation fold: " + fold);
			ArrayList<XLog> logsCV = splitTrainingTest(context,splitLogs,fold);
			
			// add to data log collection
			output.addLog(logsCV.get(0));
			output.addLog(logsCV.get(1));
			output.addLog(logsCV.get(2));
			
			// add training/test logs to workspace
			context.getProvidedObjectManager().createProvidedObject("training log " + fold, 
					logsCV.get(0), XLog.class , context);
			org.processmining.plugins.utils.ProvidedObjectHelper.setFavorite(context, logsCV.get(1));
			context.getProvidedObjectManager().createProvidedObject("fitting test log " + fold, 
					logsCV.get(1), XLog.class , context);
			org.processmining.plugins.utils.ProvidedObjectHelper.setFavorite(context, logsCV.get(1));
			context.getProvidedObjectManager().createProvidedObject("non-fitting log " + fold, 
					logsCV.get(2), XLog.class , context);
			org.processmining.plugins.utils.ProvidedObjectHelper.setFavorite(context, logsCV.get(2));
		}
		
		return output;
	}
	
	private static ArrayList<XLog> splitLog(PluginContext context, XLog log, int noFittingCases) {
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog fitting = factory.createLog((XAttributeMap) log.getAttributes().clone());
		XLog nonfitting = factory.createLog((XAttributeMap) log.getAttributes().clone());
		fitting.getClassifiers().addAll(log.getClassifiers());
		fitting.getExtensions().addAll(log.getExtensions());
		fitting.getGlobalTraceAttributes().addAll(log.getGlobalTraceAttributes());
		fitting.getGlobalEventAttributes().addAll(log.getGlobalEventAttributes());
		nonfitting.getClassifiers().addAll(log.getClassifiers());
		nonfitting.getExtensions().addAll(log.getExtensions());
		nonfitting.getGlobalTraceAttributes().addAll(log.getGlobalTraceAttributes());
		nonfitting.getGlobalEventAttributes().addAll(log.getGlobalEventAttributes());
		for (XTrace trace : log) {
			for (String key : trace.getAttributes().keySet()) {
				String value = trace.getAttributes().get(key).toString();
				if (Integer.parseInt(value) < noFittingCases + 1) {
					fitting.add(trace);
				}
				else
					nonfitting.add(trace);
			}
		}
		XConceptExtension.instance().assignName(fitting, "fitting");
		XConceptExtension.instance().assignName(nonfitting, "non-fitting");
		ArrayList<XLog> logs = new ArrayList<XLog>();
		logs.add(fitting);
		logs.add(nonfitting);
		return logs;
	}
	
	private static ArrayList<XLog> splitTrainingTest(PluginContext context, ArrayList<XLog> logs, int run) {
		XFactory factory = XFactoryRegistry.instance().currentDefault();
		XLog training = factory.createLog((XAttributeMap) logs.get(0).getAttributes().clone());
		XLog testFitting = factory.createLog((XAttributeMap) logs.get(0).getAttributes().clone());
		XLog testNonFitting = factory.createLog((XAttributeMap) logs.get(0).getAttributes().clone());
		training.getClassifiers().addAll(logs.get(0).getClassifiers());
		training.getExtensions().addAll(logs.get(0).getExtensions());
		training.getGlobalTraceAttributes().addAll(logs.get(0).getGlobalTraceAttributes());
		training.getGlobalEventAttributes().addAll(logs.get(0).getGlobalEventAttributes());
		testFitting.getClassifiers().addAll(logs.get(1).getClassifiers());
		testFitting.getExtensions().addAll(logs.get(1).getExtensions());
		testFitting.getGlobalTraceAttributes().addAll(logs.get(1).getGlobalTraceAttributes());
		testFitting.getGlobalEventAttributes().addAll(logs.get(1).getGlobalEventAttributes());
		testNonFitting.getClassifiers().addAll(logs.get(1).getClassifiers());
		testNonFitting.getExtensions().addAll(logs.get(1).getExtensions());
		testNonFitting.getGlobalTraceAttributes().addAll(logs.get(1).getGlobalTraceAttributes());
		testNonFitting.getGlobalEventAttributes().addAll(logs.get(1).getGlobalEventAttributes());
		// add fitting traces
		int startIndex;
		int endIndex = (logs.get(0).size()/10)*run;
		if (run == 1)
			startIndex = 1;
		else
			startIndex = (logs.get(0).size()/10)*(run-1) + 1;
		int added = 0;
		// System.out.println("Fitting log: start index " + startIndex + ", end index " + endIndex);
		for (XTrace trace : logs.get(0)) {
			for (String key : trace.getAttributes().keySet()) {
				String value = trace.getAttributes().get(key).toString();
				if (Integer.parseInt(value) >= startIndex && Integer.parseInt(value) <= endIndex) {
					// only add first half of those traces:
					if (added < logs.get(0).size()/10/2) {
						testFitting.add(trace);
						added++;
						// System.out.println("added fitting test trace " + value);
					}
				}
				else {
					training.add(trace);
					// System.out.println("added training trace " + value);
				}
			}
		}
		// add non-fitting traces
		if (run == 1)
			startIndex = logs.get(0).size() + 1;
		else
			startIndex = logs.get(0).size() + logs.get(1).size()/10*(run-1) + 1;
		endIndex = logs.get(0).size() + (logs.get(1).size()/10)*run;
		// System.out.println("Non-fitting log: start index " + startIndex + ", end index " + endIndex);
		for (XTrace trace : logs.get(1)) {
			for (String key : trace.getAttributes().keySet()) {
				String value = trace.getAttributes().get(key).toString();
				if (Integer.parseInt(value) >= startIndex && Integer.parseInt(value) <= endIndex) {
					testNonFitting.add(trace);
					// System.out.println("added non-fitting test trace " + value);
				}
			}
		}
		ArrayList<XLog> logsCV = new ArrayList<XLog>();
		logsCV.add(training);
		logsCV.add(testFitting);
		logsCV.add(testNonFitting);
		return logsCV;
	}
}
