package org.processmining.ptandloggenerator.plugins.data;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import javax.swing.JOptionPane;

import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.datadiscovery.estimators.DecisionTreeBasedFunctionEstimator;
import org.processmining.datadiscovery.estimators.FunctionEstimation;
import org.processmining.datadiscovery.estimators.FunctionEstimator;
import org.processmining.datadiscovery.estimators.Type;
import org.processmining.datadiscovery.estimators.impl.DecisionTreeFunctionEstimator;
import org.processmining.datadiscovery.estimators.impl.OverlappingEstimatorLocalDecisionTree;
import org.processmining.datadiscovery.model.DecisionPointResult;
import org.processmining.datadiscovery.model.DiscoveredPetriNetWithData;
import org.processmining.datadiscovery.plugins.DecisionMining;
import org.processmining.datadiscovery.plugins.alignment.TraceProcessor;
import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.util.ui.widgets.ProMComboCheckBox;
import org.processmining.log.utils.XUtils;
import org.processmining.models.connections.petrinets.behavioral.FinalMarkingConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithDataFactory;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.astar.petrinet.PetrinetReplayerWithILP;
import org.processmining.plugins.balancedconformance.config.BalancedProcessorConfiguration;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.finalmarkingprovider.MarkingEditorPanel;
import org.processmining.plugins.petrinet.replayer.PNLogReplayer;
import org.processmining.plugins.petrinet.replayer.algorithms.costbasedcomplete.CostBasedCompleteParam;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;
import org.processmining.plugins.utils.ProvidedObjectHelper;
import com.google.common.util.concurrent.AtomicLongMap;

public class DecisionMinerNoUI {
	
	public DecisionMinerNoUI() {	
	}
	
	public static Object[] decisionDiscovery(final PluginContext context, Petrinet net,
			final XLog log, boolean useOverlapping) throws Exception {
		/*
		 * Setup multi-threading for later use
		 */
		int maxConcurrentThreads = Runtime.getRuntime().availableProcessors();
		ThreadPoolExecutor pool = new ThreadPoolExecutor(maxConcurrentThreads, maxConcurrentThreads, 60,
				TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>());
		pool.allowCoreThreadTimeOut(false);

		Future<Map<String, Type>> classTypesFuture = pool.submit(new Callable<Map<String, Type>>() {

			public Map<String, Type> call() throws Exception {
				/*
				 * Store the log's Attributes and their Data Types in
				 * Map<String, Type> classTypes
				 */
				return DecisionMining.extractAttributeInformation(log);
			}
		});
		
		Future<Map<String, Set<String>>> literalValuesFuture = pool.submit(new Callable<Map<String, Set<String>>>() {

			public Map<String, Set<String>> call() throws Exception {

				/*
				 * Extract the literal values for the attributes of type
				 * XAttributeLiteral and store them in literalValues.
				 * literalValues is a mapping from attribute name to a set of
				 * literal string values for that attribute.
				 */
				return DecisionMining.getLiteralValuesMap(log);
			}
		});
		
		// replay the log on the net
		PNRepResult input = null;
		
		// default mapping from balanced alignment
		XEventClasses eventClasses = XUtils.createEventClasses(log.getClassifiers().get(0), log);
		TransEvClassMapping activityMapping = BalancedProcessorConfiguration.createDefaultMappingTransitionsToEventClasses(net,
				eventClasses.getClassifier(), eventClasses);
		XLogInfo logInfo = XLogInfoFactory.createLogInfo(log, activityMapping.getEventClassifier());
		CostBasedCompleteParam parameters = new CostBasedCompleteParam(logInfo.getEventClasses().getClasses(),
				activityMapping.getDummyEventClass(), net.getTransitions(), 1, 1);
		parameters.setGUIMode(false);
		parameters.setCreateConn(false);
		
		Marking initMarking = null;
		try {
			initMarking = context.getConnectionManager()
					.getFirstConnection(InitialMarkingConnection.class, context, net)
					.getObjectWithRole(FinalMarkingConnection.MARKING);
			if (initMarking.isEmpty()) {
				int yn = JOptionPane.showConfirmDialog(null, "The initial marking is empty. Is this intended?",
						"Initial Marking", JOptionPane.YES_NO_OPTION);
				if (yn == JOptionPane.NO_OPTION) {
					JOptionPane.showMessageDialog(null,
							"Please create the correct initial marking using the appropriate plug-in");
					return null;
				}
			}
		} catch (Exception e) {
			initMarking = createMarking((UIPluginContext) context, net, "Initial Marking");
			ProvidedObjectHelper.publish(context, "Initial Marking for " + net.getLabel(), initMarking, Marking.class,
					false);
			context.getConnectionManager().addConnection(new InitialMarkingConnection(net, initMarking));
		}
		parameters.setInitialMarking(initMarking);

		Marking finalMarking = null;
		try {
			finalMarking = context.getConnectionManager().getFirstConnection(FinalMarkingConnection.class, context, net)
					.getObjectWithRole(FinalMarkingConnection.MARKING);
			if (finalMarking.isEmpty()) {
				int yn = JOptionPane.showConfirmDialog(null, "The final marking is empty. Is this intended?",
						"Final Marking", JOptionPane.YES_NO_OPTION);
				if (yn == JOptionPane.NO_OPTION) {
					JOptionPane.showMessageDialog(null,
							"Please create the correct final marking using the appropriate plug-in");
					return null;
				}
			}

		} catch (Exception e) {
			finalMarking = createMarking((UIPluginContext) context, net, "Final Marking");
			ProvidedObjectHelper.publish(context, "Final Marking for " + net.getLabel(), finalMarking, Marking.class,
					false);
			context.getConnectionManager().addConnection(new FinalMarkingConnection(net, finalMarking));
		}
		parameters.setFinalMarkings(new Marking[] { finalMarking });
		
//		parameters.setInitialMarking(net.initialMarking);
//		parameters.setFinalMarkings(new Marking[] { net.finalMarking });
		
		parameters.setMaxNumOfStates(Integer.MAX_VALUE);

		// Enable multi-threaded alignments, but do not use too much threads to avoid memory issues
		parameters.setNumThreads((int) (maxConcurrentThreads * 0.75));
		
		input = new PNLogReplayer().replayLog(context, net, log, activityMapping,
				new PetrinetReplayerWithILP(), parameters);
		
		if (input == null) {
			context.getFutureResult(0).cancel(true); // Cancel the Activity in ProM and
			return null; // Return nothing			
		}
		
		Map<String, Type> classTypes = classTypesFuture.get();
		Map<String, Set<String>> literalValues = literalValuesFuture.get();

		/*
		 * Initialize a HashMap<Place, FunctionEstimator>, linking a
		 * FunctionEstimator to each Place
		 */
		Map<Place, FunctionEstimator> estimators = new HashMap<>();

		/*
		 * Initialize a thread-safe AtomicLongMap<Transition> denoting the
		 * numberOfExecutions per Transition
		 */
		AtomicLongMap<Transition> numberOfExecutions = AtomicLongMap.create();

		/*
		 * Initialize a HashMap<Transition, Map<String, Integer>> denoting the
		 * number of writes per Transition FM: Does not need to be thread-safe!!
		 * Only get is executed!
		 */
		Map<Transition, AtomicLongMap<String>> numberOfWritesPerTransition = new HashMap<>();

		for (Transition trans : net.getTransitions()) {
			/*
			 * For each transition, set the numberOfExecutions to 0 and
			 * initialize the HashMap<String, Integer>
			 */
			numberOfExecutions.put(trans, 0);
			numberOfWritesPerTransition.put(trans, AtomicLongMap.<String>create());
		}
		
		/*
		 * Given the PNRepResult, find the Fitness to initialize the
		 * fitnessThreshold
		 */
		Double initialFitness = ((Double) input.getInfo().get(PNRepResult.TRACEFITNESS));
		
		// fix places in the decision tree panel, it's a GUESS!
		Place[] consideredPlaces = net.getPlaces().toArray(new Place[net.getPlaces().size()]);
		Arrays.sort(consideredPlaces);
		ProMComboCheckBox placesCCBox = new ProMComboCheckBox(consideredPlaces, true);
		
		/*
		 * For each place with at least 2 outgoing edges in the net ..
		 */
		for (Place place : net.getPlaces()) {
			// If 'place' does not represent an OR-split (outgoing edges < 2), goto next iteration of for-loop (skip the place, not interesting)
			if (net.getOutEdges(place).size() < 2) {
				continue;
			}

			if (!placesCCBox.getSelectedItems().contains(place)) {
				continue;
			}
			/*
			 * 'place' is part of an OR-split decision point with >= 2 outgoing
			 * edges. Prepare an array outputValues[] to collect the target
			 * Transitions of the outgoing edges of 'place'
			 */
			Transition outputValues[] = new Transition[net.getOutEdges(place).size()];
			int index = 0;
			/*
			 * For each outgoing edge 'arc' of Place 'place', store that edge's
			 * target Transition in the array.
			 */
			for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> arc : net.getOutEdges(place)) {
				outputValues[index++] = (Transition) arc.getTarget();
			}

			/*
			 * Prepare variable f for a functionEstimator, and create one with
			 * the attribute types, values and resulting Transitions for Place
			 * place
			 */
			FunctionEstimator f;
			
			// apply "basic decision tree" or "overlapping rules"
			if (useOverlapping) {
				f = new OverlappingEstimatorLocalDecisionTree(classTypes, literalValues, outputValues, log.size(),
				place.getLabel());
			}
			else {
				f = new DecisionTreeFunctionEstimator(classTypes, literalValues, outputValues, place.getLabel(),
						log.size());
			}
			
			/*
			 * Associate the created FunctionEstimator 'f' with Place 'place' by
			 * putting them in a Mapping from <Place> to <FunctionEstimator>
			 */
			estimators.put(place, f);
		}
		
		// Prepare context progress bar
		final Progress progress = context.getProgress();
		progress.setMaximum(log.size() + estimators.entrySet().size());
		progress.setValue(0);
		context.log("Processing the log traces...");
		
		List<Future<Integer>> traceFutures = new ArrayList<>();

		// Prepare trace counters
		int traceSkipped = 0;
		int totalNumTraces = 0;
		// For each alignment in the PNRepResult 'input'
		for (SyncReplayResult alignment : input) {
			// Count the number of traces encountered in total
			totalNumTraces += alignment.getTraceIndex().size();
			// If the alignment's fitness complies with the specified minimum fitness in fitnessThresholdSlider
			if (alignment.getInfo().get(PNRepResult.TRACEFITNESS).floatValue() >= initialFitness) {
				// Then for each trace in the alignment
				for (Integer index : alignment.getTraceIndex()) {
					/*
					 * Have a thread in the pool process the trace in the
					 * alignment, adding instances to the estimator and keeping
					 * track of attribute values. For each alignment, For each
					 * step in the alignment, For each in-edge of the
					 * transition, If the source-Place of that in-edge has an
					 * estimator, Add an instance corresponding to the variable
					 * values in that place, before executing the transition
					 * corresponding to the step. An instance is a set of
					 * attribute values' pre-values and a transition to be
					 * executed
					 */
					traceFutures.add(pool.submit(new TraceProcessor(net, log.get(index), estimators, alignment,
							numberOfExecutions, numberOfWritesPerTransition, progress), index));
				}
			} else {
				// The alignment's fitness is lower than the fitness threshold; skip the trace and count the skipped traces
				traceSkipped += alignment.getTraceIndex().size();
			}
		}
		
		// Report # of skipped traces, shut down the thread pool
		context.log("Skipped " + traceSkipped + " low-fitting traces out of " + totalNumTraces);

		for (Future<Integer> traceFuture : traceFutures) {
			// This blocks until the trace processor is done
			traceFuture.get();
			if (progress.isCancelled()) {
				context.getFutureResult(0).cancel(true);
				return null;
			}

		}
		
		DiscoveredPetriNetWithData discoveredDPN = null;
		Map<Place, Future<DecisionPointResult>> results = new HashMap<>();

		// For each place
		for (Place place : net.getPlaces()) {
			// Configure the corresponding function estimator's decision tree parameters
			FunctionEstimator f = estimators.get(place);
			if (f != null) {
				if (f instanceof DecisionTreeBasedFunctionEstimator) {
					// Set the minimal number of instances per leaf, in per mille (relative to numInstances)
					((DecisionTreeBasedFunctionEstimator) f)
							.setMinNumObj(20);
					// Prune the tree?
					((DecisionTreeBasedFunctionEstimator) f).setUnpruned(false);
					// Binary split?
					((DecisionTreeBasedFunctionEstimator) f).setBinarySplit(true);
					((DecisionTreeBasedFunctionEstimator) f).setCrossValidate(true);
				}
			}
		}
		
		/*
		 * Collect result for each decision point
		 */
		for (Entry<Place, FunctionEstimator> estimatorPlacePair : estimators.entrySet()) {

			final Place place = estimatorPlacePair.getKey();
			final FunctionEstimator f = estimatorPlacePair.getValue();

			Future<DecisionPointResult> result = pool.submit(new Callable<DecisionPointResult>() {

				public DecisionPointResult call() throws Exception {

					// Calculate the conditions with likelihoods for each target transition of place entry2.getKey()
					final Map<Object, FunctionEstimation> estimationTransitionExpression = f
							.getFunctionEstimation(null);
					
					double sumFMeasures = 0;
					int numRules = 0;
					for (FunctionEstimation val : estimationTransitionExpression.values()) {
						sumFMeasures += val.getQualityMeasure();
						numRules++;
					}

					final double singleFScore = numRules > 0 ? sumFMeasures / numRules : 0;
					final String decisionPointClassifier = f.toString();

					DecisionPointResult result = new DecisionPointResult() {

						public String toString() {
							return decisionPointClassifier;
						}

						public double getQualityMeasure() {
							return singleFScore;
						}

						public Map<Object, FunctionEstimation> getEstimatedGuards() {

							return estimationTransitionExpression;
						}

					};

					if (result.getQualityMeasure() > 0) {
						context.log(String.format("Generated the conditions for decision point %s with f-score %s",
								place.getLabel(), result.getQualityMeasure()));
					}
					progress.inc();

					return result;
				}

			});
			results.put(place, result);
		}
		
		/*
		 * Prepare the mining algorithm's resulting PetriNetWithData.
		 */
		String dpnName = "discovered dpn";
		final PetriNetWithDataFactory factory = new PetriNetWithDataFactory(net,
				new DiscoveredPetriNetWithData(dpnName), false);
		discoveredDPN = (DiscoveredPetriNetWithData) factory.getRetValue(); // cast if safe
		
		/*
		 * For each entry in classTypes, <String, Type> representing (Attribute
		 * name, attribute type), depending on the type add a new Variable to
		 * the new PetriNetWithData without min or max values
		 */

		for (Entry<String, Type> entry : classTypes.entrySet()) {
			Class<?> classType = null;
			switch (entry.getValue()) {
				case BOOLEAN :
					classType = Boolean.class;
					break;
				case CONTINUOS :
					classType = Double.class;
					break;
				case DISCRETE :
					classType = Long.class;
					break;
				case LITERAL :
					classType = String.class;
					break;
				case TIMESTAMP :
					classType = Date.class;
					break;
				default :
					break;

			}
			String wekaUnescaped = DecisionMining.wekaUnescape(entry.getKey());
			String saneVariableName = GuardExpression.Factory.transformToVariableIdentifier(wekaUnescaped);
			discoveredDPN.addVariable(saneVariableName, classType, null, null);
		}
		
		//TODO FM, we should not change the default Locale!! What about concurrent operations that rely on the correct Locale? Why is it done anyway?
		Locale defaultLocale = Locale.getDefault();
		Locale.setDefault(Locale.US);

		double sumFScores = 0;
		int numEstimators = 0;

		Map<Place, Place> placeMapping = factory.getPlaceMapping();

		for (Entry<Place, Future<DecisionPointResult>> futureEntry : results.entrySet()) {
			try {
				DecisionPointResult result = futureEntry.getValue().get();
				discoveredDPN.storeDecisionPointResult(placeMapping.get(futureEntry.getKey()), result);

				// If, for any Transition at this decision point, an expression is found..
				if (!result.getEstimatedGuards().isEmpty()) {
					// Then, for each such Transition, set the guard in the PetriNetWithData
					for (Entry<Object, FunctionEstimation> transitionEntry : result.getEstimatedGuards().entrySet()) {
						Transition transitionInPNWithoutData = (Transition) transitionEntry.getKey();
						PNWDTransition transitionInPNWithData = (PNWDTransition) factory.getTransMapping()
								.get(transitionInPNWithoutData);
						FunctionEstimation value = transitionEntry.getValue();
						if (transitionInPNWithData.getGuardExpression() != null) {
							//TODO find correct method to update f-score / what to do in the general case for more than two incoming arcs
							Double combinedFScore = (value.getQualityMeasure() + transitionInPNWithData.getQuality())
									/ 2;
							GuardExpression existingGuard = transitionInPNWithData.getGuardExpression();
							GuardExpression additionalGuard = value.getExpression();
							GuardExpression combinedGuard = GuardExpression.Operation.and(existingGuard,
									additionalGuard);
							context.log(
									String.format("Combining two guards for non-free choice construct: %s (%s) %s (%s)",
											existingGuard, value.getQualityMeasure(), additionalGuard,
											transitionInPNWithData.getQuality()));
							discoveredDPN.setGuard(transitionInPNWithData, combinedGuard, combinedFScore);
						} else {
							discoveredDPN.setGuard(transitionInPNWithData, value.getExpression(),
									value.getQualityMeasure());
						}
					}

					sumFScores += result.getQualityMeasure();
					numEstimators++;
				}
			} catch (ExecutionException e) {
				context.log(e);
			}
		}
		
		//TODO see above!
		Locale.setDefault(defaultLocale);

		if (sumFScores > 0) {
			context.log("Average F Score: " + sumFScores / numEstimators);
		}

		for (Transition transitionInPNWithoutData : net.getTransitions()) {
			PNWDTransition transitionInPNWithData = (PNWDTransition) factory.getTransMapping()
					.get(transitionInPNWithoutData);

			if (true && !transitionInPNWithData.isInvisible()) {

				//Set the read operations
				if (transitionInPNWithData.getGuardExpression() != null) {
					// Only use normal variables as those are read
					Set<String> normalVariables = transitionInPNWithData.getGuardExpression().getNormalVariables();
					for (String varName : normalVariables) {
						discoveredDPN.assignReadOperation(transitionInPNWithData, discoveredDPN.getVariable(varName));
					}
				}

				//Set the write operations
				long numberOfExecution = numberOfExecutions.get(transitionInPNWithoutData);
				for (Entry<String, Long> numWritesVariable : numberOfWritesPerTransition.get(transitionInPNWithoutData)
						.asMap().entrySet()) {
					if (numWritesVariable.getValue() > (numberOfExecution * 66) / 100) {
						DataElement dataElem = discoveredDPN.getVariable(numWritesVariable.getKey());
						if (dataElem != null)
							discoveredDPN.assignWriteOperation(transitionInPNWithData, dataElem);
					}
				}
			}
		}
		
//		if (removeAttribInNoGuardBox.isSelected())
//			discoveredDPN.removeAllVariablesNotInGuard();

		Marking[] markings = factory.cloneInitialAndFinalConnection(context);

		// Set appropriate name for returned DPN
		context.getFutureResult(0).setLabel(dpnName);

		pool.shutdown();

		return new Object[] { discoveredDPN, markings[0], markings[1] };
	}
	
	private static Marking createMarking(UIPluginContext context, PetrinetGraph net, String title) {
		MarkingEditorPanel editor = new MarkingEditorPanel(title);

		return editor.getMarking(context, net);
	}

}
