package org.processmining.ptandloggenerator.plugins.data;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.ptandloggenerator.algorithms.DataLogCollectionGenerator;
import org.processmining.ptandloggenerator.dialogs.dataLogSettings;
import org.processmining.ptandloggenerator.models.DataLogCollection;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;

@Plugin(name = "Generate Data Log Collection from Process Trees", parameterLabels = { "Newick Tree Collection", "Log Generator Parameters" }, returnLabels = {
"Data log collection (XES format)" }, returnTypes = { DataLogCollection.class }, 
help = "Generate events logs for a given sample of process trees. URI: http://hdl.handle.net/1942/20818")
public class GenerateDataLogCollection {
	
	@UITopiaVariant(affiliation = "Hasselt University", author = "Toon Jouck", email = "toon.jouck@uhasselt.be")
	@PluginVariant(variantLabel = "Generate Event Logs (XES) with Data from Process Trees (Newick Trees)", requiredParameterLabels = { 0 })
	public DataLogCollection run(UIPluginContext context, NewickTreeCollection collection) {
		
		//set maximum for progressbar
		context.getProgress().setMaximum(collection.getNewickTreeList().size());
		context.getProgress().setValue(0);
		
		context.log("Creating log settings");
		//get inputs from dialog
		dataLogSettings settings = new dataLogSettings();
		
		//show input wizard
		InteractionResult result = context.showWizard("Data log generator Settings", true, true, settings.initComponents());
		
		switch (result) {
			case CANCEL :
				return cancel(context, collection, "The user has cancelled the data log generator!");
			case FINISHED :
				settings.setNoCases();
				settings.setNoNoisyCases();
				settings.setMaxInput();
				settings.setMaxCutoff();
				settings.setTargetDeterminism();
				context.log("Starting log generation");
				DataLogCollectionGenerator generator = new DataLogCollectionGenerator(collection,
						settings.getNoCases(),settings.getNoNoisyCases(),settings.getMaxInput(),
						settings.getMaxCutoff(),settings.getTargetDeterminism());
				int i = 1;
				
				for (XLog log: generator.getLogs().getXLogCollection()){
					//unpack created log
					context.getProvidedObjectManager().createProvidedObject("Log " + Integer.toString(i), log, XLog.class , context);
					i++;
					org.processmining.plugins.utils.ProvidedObjectHelper.setFavorite(context, log);
					
					//advance the progressbar
					context.getProgress().inc();
				}
				return generator.getLogs();
			default :
				System.out.println("problem with the settings");
				return null;
		}
	}
	
	/**
	* Cancel the future results, display a error message and return all nulls.
	* @param context Context of the plug-in.
	* @param msg Error message.
	* @return Return an object array of nulls;
	*/
	private DataLogCollection cancel(PluginContext context, NewickTreeCollection collection, String msg) {
		context.log(msg);
		context.getFutureResult(0).cancel(true);
		context.getFutureResult(1).cancel(true);
		return new DataLogCollectionGenerator(collection,"0","0","0","0","0").getLogs();
	}
}
