package org.processmining.ptandloggenerator.renderers;

import javax.swing.JComponent;
import javax.swing.JTabbedPane;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.processtree.arrays.ProcessTreeArray;
import org.processmining.processtree.arrays.impl.ProcessTreeArrayImpl;
import org.processmining.processtree.visualization.bpmn.BPMNVisualization;
import org.processmining.processtree.visualization.tree.TreeVisualization;
import org.processmining.ptandloggenerator.models.NewickTree;
import org.processmining.ptandloggenerator.models.NewickTreeCollection;
import org.processmining.ptandloggenerator.plugins.NewickTreeToProcessTreeConverter;

public class NewickAsProcessTreeList {
	
	@Plugin(name = "Visualize As Process Trees",
			parameterLabels = { "NewickTreeCollection" },
			returnLabels = {"Process Tree Array" },
			returnTypes = { JComponent.class })
	@Visualizer
	
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context,NewickTreeCollection input){
		ProcessTreeArray array = new ProcessTreeArrayImpl();
		NewickTreeToProcessTreeConverter converter = new NewickTreeToProcessTreeConverter();
		
		for (NewickTree tree : input.getNewickTreeList())
			array.addElement(converter.run(context, tree));

		JTabbedPane mainPane = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
		TreeVisualization treeVis = new TreeVisualization();
		for(int i = 0; i < array.getSize(); i += 100){
			JTabbedPane subTab = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
			for(int j = i; j < Math.min(i + 100, array.getSize()); j++){
				subTab.addTab(array.getElement(j).getName() + "-" + (j+1), treeVis.visualizeProperties(context, array.getElement(j).toTree()));
			}
			mainPane.addTab("" + (i+1) + "-" + (i + 100), subTab);
		}
		return mainPane;
	}
	
	
	@Plugin(name = "Visualize Equivalent BPMN", 
			returnLabels = { "Visualised Process Array" }, 
			returnTypes = { JComponent.class }, 
			parameterLabels = { "NewickTreeCollection" }, 
			userAccessible = true)
	@Visualizer
	
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualizeBPMN(PluginContext context, NewickTreeCollection input){
		ProcessTreeArray array = new ProcessTreeArrayImpl();
		NewickTreeToProcessTreeConverter converter = new NewickTreeToProcessTreeConverter();
		
		for (NewickTree tree : input.getNewickTreeList())
			array.addElement(converter.run(context, tree));
		
		JTabbedPane mainPane = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
		BPMNVisualization bpmnVis = new BPMNVisualization();
		for(int i = 0; i < array.getSize(); i += 100){
			JTabbedPane subTab = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
			for(int j = i; j < Math.min(i + 100, array.getSize()); j++){
				subTab.addTab(array.getElement(j).getName() + "-" + (j+1), bpmnVis.visualize(context, array.getElement(j)));
			}
			mainPane.addTab("" + (i+1) + "-" + (i + 100), subTab);
		}
		return mainPane;
	}

}
